@@ -3,14 +3,19 @@ from __future__ import annotations
 import contextlib
 import inspect
 import signal
+import sys
 import threading
-from typing import TYPE_CHECKING, AsyncIterator, Callable, Iterator
+import weakref
+from collections.abc import AsyncIterator, Callable, Iterator
+from typing import TYPE_CHECKING, TypeVar
 
 import outcome
 import pytest
 
 from trio.testing import RaisesGroup
 
+from .tutil import gc_collect_harder
+
 try:
     from async_generator import async_generator, yield_
 except ImportError:  # pragma: no cover
@@ -18,16 +23,24 @@ except ImportError:  # pragma: no cover
 
 from ... import _core
 from ..._abc import Instrument
+from ..._core import _ki
 from ..._timeouts import sleep
-from ..._util import signal_raise
 from ...testing import wait_all_tasks_blocked
 
 if TYPE_CHECKING:
+    from collections.abc import (
+        AsyncGenerator,
+        AsyncIterator,
+        Callable,
+        Generator,
+        Iterator,
+    )
+
     from ..._core import Abort, RaiseCancelT
 
 
 def ki_self() -> None:
-    signal_raise(signal.SIGINT)
+    signal.raise_signal(signal.SIGINT)
 
 
 def test_ki_self() -> None:
@@ -152,7 +165,7 @@ async def test_generator_based_context_manager_throw() -> None:
 async def test_async_generator_agen_protection() -> None:
     @_core.enable_ki_protection
     @async_generator  # type: ignore[misc] # untyped generator
-    async def agen_protected1() -> None:
+    async def agen_protected1() -> None:  # type: ignore[misc] # untyped generator
         assert _core.currently_ki_protected()
         try:
             await yield_()
@@ -161,7 +174,7 @@ async def test_async_generator_agen_protection() -> None:
 
     @_core.disable_ki_protection
     @async_generator  # type: ignore[misc] # untyped generator
-    async def agen_unprotected1() -> None:
+    async def agen_unprotected1() -> None:  # type: ignore[misc] # untyped generator
         assert not _core.currently_ki_protected()
         try:
             await yield_()
@@ -171,7 +184,7 @@ async def test_async_generator_agen_protection() -> None:
     # Swap the order of the decorators:
     @async_generator  # type: ignore[misc] # untyped generator
     @_core.enable_ki_protection
-    async def agen_protected2() -> None:
+    async def agen_protected2() -> None:  # type: ignore[misc] # untyped generator
         assert _core.currently_ki_protected()
         try:
             await yield_()
@@ -180,7 +193,7 @@ async def test_async_generator_agen_protection() -> None:
 
     @async_generator  # type: ignore[misc] # untyped generator
     @_core.disable_ki_protection
-    async def agen_unprotected2() -> None:
+    async def agen_unprotected2() -> None:  # type: ignore[misc] # untyped generator
         assert not _core.currently_ki_protected()
         try:
             await yield_()
@@ -515,3 +528,178 @@ def test_ki_with_broken_threads() -> None:
         _core.run(inner)
     finally:
         threading._active[thread.ident] = original  # type: ignore[attr-defined]
+
+
+_T = TypeVar("_T")
+
+
+def _identity(v: _T) -> _T:
+    return v
+
+
+@pytest.mark.xfail(
+    strict=True,
+    raises=AssertionError,
+    reason=(
+        "it was decided not to protect against this case, see discussion in: "
+        "https://github.com/python-trio/trio/pull/3110#discussion_r1802123644"
+    ),
+)
+async def test_ki_does_not_leak_across_different_calls_to_inner_functions() -> None:
+    assert not _core.currently_ki_protected()
+
+    def factory(enabled: bool) -> Callable[[], bool]:
+        @_core.enable_ki_protection if enabled else _identity
+        def decorated() -> bool:
+            return _core.currently_ki_protected()
+
+        return decorated
+
+    decorated_enabled = factory(True)
+    decorated_disabled = factory(False)
+    assert decorated_enabled()
+    assert not decorated_disabled()
+
+
+async def test_ki_protection_check_does_not_freeze_locals() -> None:
+    class A:
+        pass
+
+    a = A()
+    wr_a = weakref.ref(a)
+    assert not _core.currently_ki_protected()
+    del a
+    if sys.implementation.name == "pypy":
+        gc_collect_harder()
+    assert wr_a() is None
+
+
+def test_identity_weakref_internals() -> None:
+    """To cover the parts WeakKeyIdentityDictionary won't ever reach."""
+
+    class A:
+        def __eq__(self, other: object) -> bool:
+            return False
+
+    a = A()
+    assert a != a
+    wr = _ki._IdRef(a)
+    wr_other_is_self = wr
+
+    # dict always checks identity before equality so we need to do it here
+    # to cover `if self is other`
+    assert wr == wr_other_is_self
+
+    # we want to cover __ne__ and `return NotImplemented`
+    assert wr != object()
+
+
+def test_weak_key_identity_dict_remove_callback_keyerror() -> None:
+    """We need to cover the KeyError in self._remove."""
+
+    class A:
+        def __eq__(self, other: object) -> bool:
+            return False
+
+    a = A()
+    assert a != a
+    d: _ki.WeakKeyIdentityDictionary[A, bool] = _ki.WeakKeyIdentityDictionary()
+
+    d[a] = True
+
+    data_copy = d._data.copy()
+    d._data.clear()
+    del a
+
+    gc_collect_harder()  # would call sys.unraisablehook if there's a problem
+    assert data_copy
+
+
+def test_weak_key_identity_dict_remove_callback_selfref_expired() -> None:
+    """We need to cover the KeyError in self._remove."""
+
+    class A:
+        def __eq__(self, other: object) -> bool:
+            return False
+
+    a = A()
+    assert a != a
+    d: _ki.WeakKeyIdentityDictionary[A, bool] = _ki.WeakKeyIdentityDictionary()
+
+    d[a] = True
+
+    data_copy = d._data.copy()
+    wr_d = weakref.ref(d)
+    del d
+    gc_collect_harder()  # would call sys.unraisablehook if there's a problem
+    assert wr_d() is None
+    del a
+    gc_collect_harder()
+    assert data_copy
+
+
+@_core.enable_ki_protection
+async def _protected_async_gen_fn() -> AsyncGenerator[None, None]:
+    yield
+
+
+@_core.enable_ki_protection
+async def _protected_async_fn() -> None:
+    pass
+
+
+@_core.enable_ki_protection
+def _protected_gen_fn() -> Generator[None, None, None]:
+    yield
+
+
+@_core.disable_ki_protection
+async def _unprotected_async_gen_fn() -> AsyncGenerator[None, None]:
+    yield
+
+
+@_core.disable_ki_protection
+async def _unprotected_async_fn() -> None:
+    pass
+
+
+@_core.disable_ki_protection
+def _unprotected_gen_fn() -> Generator[None, None, None]:
+    yield
+
+
+async def _consume_async_generator(agen: AsyncGenerator[None, None]) -> None:
+    try:
+        with pytest.raises(StopAsyncIteration):
+            while True:
+                await agen.asend(None)
+    finally:
+        await agen.aclose()
+
+
+def _consume_function_for_coverage(
+    fn: Callable[[], object],
+) -> None:
+    result = fn()
+    if inspect.isasyncgen(result):
+        result = _consume_async_generator(result)
+
+    assert inspect.isgenerator(result) or inspect.iscoroutine(result)
+    with pytest.raises(StopIteration):
+        while True:
+            result.send(None)
+
+
+def test_enable_disable_ki_protection_passes_on_inspect_flags() -> None:
+    assert inspect.isasyncgenfunction(_protected_async_gen_fn)
+    _consume_function_for_coverage(_protected_async_gen_fn)
+    assert inspect.iscoroutinefunction(_protected_async_fn)
+    _consume_function_for_coverage(_protected_async_fn)
+    assert inspect.isgeneratorfunction(_protected_gen_fn)
+    _consume_function_for_coverage(_protected_gen_fn)
+    assert inspect.isasyncgenfunction(_unprotected_async_gen_fn)
+    _consume_function_for_coverage(_unprotected_async_gen_fn)
+    assert inspect.iscoroutinefunction(_unprotected_async_fn)
+    _consume_function_for_coverage(_unprotected_async_fn)
+    assert inspect.isgeneratorfunction(_unprotected_gen_fn)
+    _consume_function_for_coverage(_unprotected_gen_fn)
